

/**
 * myEpideMobile. Aplicación de Gestión de información epidemiológica en emergencias internacionales.
 *
 * Copyright (C) 2010 Daniel Mayor Azcona daniel.mayor@unavarra.es
 *
 * Este programa es software libre. Puede redistribuirlo y/o modificarlo bajo los términos de la Licencia
 * Pública General de GNU según es publicada por la Free Software Foundation, bien de la versión 2 de dicha
 * Licencia o bien (según su elección) de cualquier versión posterior.
 *
 * Este programa se distribuye con la esperanza de que sea útil, pero SIN NINGUNA GARANTÍA, incluso sin la
 * garantía MERCANTIL implícita o sin garantizar la CONVENIENCIA PARA UN PROPÓSITO PARTICULAR. Véase la Licencia
 * Pública General de GNU para más detalles.
 *
 * Debería haber recibido una copia de la Licencia Pública General junto con este programa. Si no ha sido así,
 * escriba a la Free Software Foundation, Inc., en 675 Mass Ave, Cambridge, MA 02139, EEUU.
 *
 */

import javax.microedition.io.Connector;
import javax.microedition.io.HttpConnection;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.OutputStream;

/**
 * Realiza las conexiones con el clúster central mediante el protocolo HTTP.
 * @author Daniel
 */
public class conectaHTTP {

    String salida = "CONNECTION FAILURE";
    DataInputStream dis = null;

    /**
     * Constructor de la clase conectaHTTP.
     * @param stringjson texto que se transmite vía POST al servidor.
     * @param urlquery consulta que se envía al servidor en formato <var>=.
     * @param urlserver URL del servidor.
     */
    public conectaHTTP(String stringjson, String urlserver, String urlquery){
        try {
            this.run(stringjson, urlserver, urlquery);
        } catch (Exception e) { e.printStackTrace(); }
    }

    /**
     * Muestra la respuesta del servidor.
     * @return mensaje de retorno del servidor.
     */
    public String retorno() {
        return salida;
    }

    /**
     * Lanza la conexión contra el servidor del clúster y envía el json por el método POST
     * @param stringjson texto que se transmite vía POST al servidor.
     * @param urlquery consulta que se envía al servidor en formato <var>=.
     * @param urlserver URL del servidor.
     */
    private void run(String stringjson, String urlserver, String urlquery) throws IOException {
        String url = urlserver;
        HttpConnection hc = null;
        OutputStream os = null;
        DataOutputStream dos = null;

        int rc;

        try {
            hc = (HttpConnection)Connector.open(url, Connector.READ_WRITE);
            hc.setRequestMethod(HttpConnection.POST);
            
            //hc.setRequestProperty("Content-Type","application/html");
            hc.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
            //hc.setRequestProperty("Content-Length", stringjson.length()+"");
            hc.setRequestProperty("User-Agent","Profile/MIDP-2.0 Configuration/CLDC-1.0");
            hc.setRequestProperty("Content-Language", "es-ES");
            
            dos = hc.openDataOutputStream();
            String consulta = urlquery+stringjson;

            //envía byte a byte la consulta vía POST
            byte[] byteRequest = consulta.getBytes();
            for (int i = 0; i < byteRequest.length; i++) {
                dos.writeByte(byteRequest[i]);
            }

            rc = hc.getResponseCode();
            if (rc != HttpConnection.HTTP_OK) {
                throw new IOException("HTTP response code: " + rc);
            }

            //lee la entrada del DataInputStream que se recibe del servidor
            dis = hc.openDataInputStream();
            StringBuffer sb = new StringBuffer();
            int b;
            while ((b = dis.read()) != -1) {
                if (b == 10)//si el byte leído es un salto de línea, cambiarlo por ;
                    sb.append("\n");
                else
                    sb.append((char)b);
            }
            salida = sb.toString();
        } catch (Exception e) { e.printStackTrace(); }
        finally {
            try {
                if (dis != null) dis.close();
                if (os != null) os.close();
                if (hc != null) hc.close();
            } catch (Exception e) { e.printStackTrace(); }
        }
    }
}