

/**
 * myEpideMobile. Aplicación de Gestión de información epidemiológica en emergencias internacionales.
 *
 * Copyright (C) 2010 Daniel Mayor Azcona daniel.mayor@unavarra.es
 *
 * Este programa es software libre. Puede redistribuirlo y/o modificarlo bajo los términos de la Licencia
 * Pública General de GNU según es publicada por la Free Software Foundation, bien de la versión 2 de dicha
 * Licencia o bien (según su elección) de cualquier versión posterior.
 *
 * Este programa se distribuye con la esperanza de que sea útil, pero SIN NINGUNA GARANTÍA, incluso sin la
 * garantía MERCANTIL implícita o sin garantizar la CONVENIENCIA PARA UN PROPÓSITO PARTICULAR. Véase la Licencia
 * Pública General de GNU para más detalles.
 *
 * Debería haber recibido una copia de la Licencia Pública General junto con este programa. Si no ha sido así,
 * escriba a la Free Software Foundation, Inc., en 675 Mass Ave, Cambridge, MA 02139, EEUU.
 *
 */

import java.io.InputStream;
import java.io.IOException;
import javax.microedition.io.Connector;
import javax.microedition.io.file.FileConnection;
import java.util.Vector;
import org.json.me.JSONObject;
import org.json.me.JSONException;

/**
 * Lee los ficheros de texto almacenados en el dispositivo móvil con los datos recogidos por el sistema.
 * @author Daniel Mayor
 */
public class getJSON {
    int c = 0;
    Vector v = new Vector();
    int exito = 0;
    StringBuffer sb;
    InputStream in;
    FileConnection fc;


    /**
     * Constructor de la clase getJSON. Abre un fichero con objetos JSON, los ordena por fecha y los mete en un vector.
     * @param fichero nombre del fichero.
     */
    public getJSON(String fichero) {
        //emulador NetBeans file:///root1/
        //String readFile = "file:///"+fichero;
        String readFile = System.getProperty("fileconn.dir.memorycard")+fichero;
        //String readFile = "file:///root1/"+fichero;
        sb = new StringBuffer();
        try {
            fc = (FileConnection) Connector.open(readFile, Connector.READ);
            try {
                in = fc.openInputStream();
            } catch (IOException ioe) { ioe.printStackTrace(); }
            while ((c = in.read()) != -1) {
                if (c == 10) { //almacena el string acumulado en una nueva posición del array cuando encuentra un LF
                    v.addElement(sb.toString());
                    sb = null;
                    sb = new StringBuffer();
                }
                else if (c == 13) {}//ignora el retorno de carro CR
                else
                    sb.append((char)c);
            }
            if (c == -1)
                if (sb.length() > 1) {
                    v.addElement(sb.toString());
                    sb = null;
                }
            fc.close();
        } catch (IOException ioe) {
            ioe.printStackTrace();
            exito = -1;
        } catch (NullPointerException npe) {
            npe.printStackTrace();
            exito = -2;
        } catch (Exception e) {
            e.printStackTrace();
            exito = -3;
        }
        v.trimToSize();
        v = sortVector(v, "date");
        exito = 1;
    }

    /**
     * Constructor de la clase getJSON. Abre un fichero con objetos JSON, los ordena por fecha y los mete en un vector.
     * @param fichero nombre del fichero.
     * @param ruta ruta del directorio del fichero que se va a leer.
     */
    public getJSON(String fichero, String ruta) {
        //String readFile = "file://"+ruta+fichero;
        String readFile = System.getProperty("fileconn.dir.memorycard")+fichero;
        //String readFile = "file:///root1/"+fichero;
        sb = new StringBuffer();
        try {
            fc = (FileConnection) Connector.open(readFile, Connector.READ);
            try {
                in = fc.openInputStream();
            } catch (IOException ioe) { ioe.printStackTrace(); }
            while ((c = in.read()) != -1) {
                if (c == 10) { //almacena el string acumulado en una nueva posición del array cuando encuentra un LF
                    v.addElement(sb.toString());
                    sb = null;
                    sb = new StringBuffer();
                }
                else if (c == 13) {}//ignora el retorno de carro CR
                else
                    sb.append((char)c);
            }
            if (c == -1)
                if (sb.length() > 1) {
                    v.addElement(sb.toString());
                    sb = null;
                }
            fc.close();
        } catch (IOException ioe) {
            ioe.printStackTrace();
            exito = -1;
        } catch (NullPointerException npe) {
            npe.printStackTrace();
            exito = -2;
        } catch (Exception e) {
            e.printStackTrace();
            exito = -3;
        }
        v.trimToSize();
        v = sortVector(v, "date");
        exito = 1;
    }

    /**
     * Devuelve el vector con los registros de los json almacenados.
     * @return v vector con los registros de los json almacenados.
     */
    public Vector getVjson() {
        return v;
    }

    /**
     * Convierte el vector v en un array de Strings.
     * @return jsons array de Strings en el que almacena los json almacenados en el vector.
     */
    public String[] jsons() {
        String[] jsons = new String[v.size()];
        for(int i=0; i<v.size(); i++){
            jsons[i] = v.elementAt(i).toString();
        }
        return jsons;
    }

    /**
     * Devuelve el tamaño del vector.
     * @return v.size() tamaño del vector.
     */
    public int n_jsons() {
        return v.size();
    }

    /**
     * Devuelve un los elementos del vector concatenados y separados por un salto de línea
     * @return cont String con los elementos del vector
     */
    public String all() {
        String cont;
        if (v.size() > 0) {
            cont = v.elementAt(0).toString();
            for(int i = 1; i < v.size(); i++) {
                cont += v.elementAt(i).toString()+"\n";
            }
        }
        else
            cont = "Fichero vacío";
        return cont;
    }

    /**
     * Ordena los registros de los json almacenados en el vector de menor a mayor
     * en función del key introducido con el algoritmo de la burbuja.
     * @param key nombre de la variable que se utiliza para realizar la ordenación.
     * @param v vector que se va a ordenar.
     * @return v_sorted vector ordenado.
     */
    public Vector sortVector(Vector v, String key) {
        JSONObject[] jsones = new JSONObject[v.size()];
        String[] dates = new String[v.size()];          //guarda las fechas
        Vector v_sorted = new Vector();
        try {
            for(int i = 0; i < v.size(); i++){
                jsones[i] = new JSONObject(v.elementAt(i).toString());
                dates[i] = jsones[i].get("date").toString();  //almacena la fecha
            }
        } catch (JSONException jse) {
            jse.printStackTrace();
        }
        ordenamientoBurbuja(dates, v.size());
        //ahora que están las fechas ordenadas
        //se va a generar el nuevo vector con las fechas ordenadas
        int j;
        try {
            for (int i = 0; i < v.size(); i++){
                j = 0;
                while (dates[i].compareTo(jsones[j].get("date").toString()) != 0) {
                    j++;
                }
                v_sorted.addElement(jsones[j]);
            }
        } catch (JSONException jse) {
            jse.printStackTrace();
        }
        v_sorted.trimToSize();
        return v_sorted;
    }

    /**
     * Algoritmo de ordenación por el método de la burbuja.
     * @param util_v número de elementos del conjunto que se va a ordenar.
     * @param v elementos que se van a ordenar.
     */
    public void ordenamientoBurbuja(String v[], int util_v) {
        String temp;
        for (int i = 0; i <= util_v - 2; i++) {
            for (int j = i + 1; j <= util_v - 1; j++) {
                if (v[i].compareTo(v[j]) > 0) {
                    temp = v[i];
                    v[i] = v[j];
                    v[j] = temp;
                }
            }
        }
    }

}