

/**
 * myEpideMobile. Aplicación de Gestión de información epidemiológica en emergencias internacionales.
 *
 * Copyright (C) 2010 Daniel Mayor Azcona daniel.mayor@unavarra.es
 *
 * Este programa es software libre. Puede redistribuirlo y/o modificarlo bajo los términos de la Licencia
 * Pública General de GNU según es publicada por la Free Software Foundation, bien de la versión 2 de dicha
 * Licencia o bien (según su elección) de cualquier versión posterior.
 *
 * Este programa se distribuye con la esperanza de que sea útil, pero SIN NINGUNA GARANTÍA, incluso sin la
 * garantía MERCANTIL implícita o sin garantizar la CONVENIENCIA PARA UN PROPÓSITO PARTICULAR. Véase la Licencia
 * Pública General de GNU para más detalles.
 *
 * Debería haber recibido una copia de la Licencia Pública General junto con este programa. Si no ha sido así,
 * escriba a la Free Software Foundation, Inc., en 675 Mass Ave, Cambridge, MA 02139, EEUU.
 *
 */

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import javax.microedition.io.Connector;
import javax.microedition.io.file.FileConnection;
import java.util.Vector;
import org.json.me.JSONObject;

/**
 * Contiene métodos para guardar los datos recogidos con el programa en ficheros de texto almacenados
 * en el dispositivo móvil.
 * @author Daniel Mayor
 */
public class saveData {
    int exito = 0;

    /**
     * Constructor de la clase saveData. Guarda un json en forma de String en un fichero de texto.
     * Tiene la opción de sobrescribir el fichero o escribir al final de éste.
     * @param fichero nombre del fichero donde se va a guardar la información.
     * @param json objeto que se va a guardar en el fichero.
     * @param sobreescribir si es true se sobreescribirá el contenido fichero. Si es false se respetará el contenido
     * y la información se añadirá al final del fichero.
     * @return código de retorno del constructor. Si finaliza con éxito devuelve 1.
     */
    public int saveData(String fichero, String json, final boolean sobreescribir) {
        final String writeFile, writeContents;
        try {
            //emulador NetBeans file:///root1/
            //writeFile = "file:///C:/"+fichero;
            writeFile = System.getProperty("fileconn.dir.memorycard")+fichero;
            //writeFile = "file:///root1/"+fichero;
            writeContents = json;
            try {
                OutputStream os;
                FileConnection fc = (FileConnection) Connector.open(writeFile, Connector.READ_WRITE);
                if (!fc.exists())
                    fc.create();
                if (sobreescribir) {
                    os = fc.openOutputStream();//empieza a escribir al principio del fichero
                }
                else
                    os = fc.openOutputStream(fc.fileSize());//le decimos que empiece al final
                PrintStream ps = new PrintStream(os);
                ps.println(writeContents);
                os.flush();
                os.close();
                fc.close();
                exito = 1;
            } catch (IOException e){
                e.printStackTrace();
                exito = -1;
            } catch (SecurityException e){
                e.printStackTrace();
                exito = -2;
            }
        }
        catch (Exception e) {
            e.printStackTrace();
            exito = -1;
        }
        return exito;
    }

    /**
     * Constructor de la clase saveData. Guarda un vector de json en forma de String en un fichero de texto.
     * Tiene la opción de sobrescribir el fichero o escribir al final de éste.
     * @param fichero nombre del fichero donde se va a guardar la información.
     * @param sobreescribir si es true se sobreescribirá el contenido fichero. Si es false se respetará el contenido
     * y la información se añadirá al final del fichero.
     * @param vjson vector cuyos elementos serán guardados en el fichero separados por saltos de línea.
     * @return código de retorno del constructor. Si finaliza con éxito devuelve 1.
     */
    public int saveData(String fichero, Vector vjson, final boolean sobreescribir) {
        final String writeFile;
        //emulador NetBeans file:///root1/
        //writeFile = "file:///C:/"+fichero;
        writeFile = System.getProperty("fileconn.dir.memorycard")+fichero;
        //writeFile = "file:///root1/"+fichero;
        try {
            OutputStream os;
            FileConnection fc = (FileConnection) Connector.open(writeFile, Connector.READ_WRITE);
            if (!fc.exists())
                fc.create();
            if (sobreescribir) {
                fc.truncate(0);//elimina el contenido del fichero registros
                os = fc.openOutputStream();
                vjson.trimToSize();
                fc.fileSize();
            }
            else
                os = fc.openOutputStream(fc.fileSize());//para que empiece a escribir al final del fichero
            PrintStream ps = new PrintStream(os);
            for (int i=0; i<vjson.size(); i++)
                ps.println(vjson.elementAt(i).toString());
            fc.close();
            exito = 1;
        }
        catch (IOException e) {
            e.printStackTrace();
            exito = -1;
        }
        catch (SecurityException se) {
            se.printStackTrace();
            exito = -2;
        }
        return exito;
    }

    /**
     * Constructor de la clase saveData. Guarda un array de objetos JSON en un fichero de texto.
     * Tiene la opción de sobrescribir el fichero o escribir al final de éste.
     * @param fichero nombre del fichero donde se va a guardar la información.
     * @param jObject array de objetos JSON cuyos elementos serán guardados en el fichero separados por saltos de línea.
     * @param sobreescribir si es true se sobreescribirá el contenido fichero. Si es false se respetará el contenido
     * y la información se añadirá al final del fichero.
     * @return código de retorno del constructor. Si finaliza con éxito devuelve 1.
     */
    public int saveData(String fichero, JSONObject[] jObject, final boolean sobreescribir) {
        final String writeFile;
        Vector v = new Vector();
        for (int i=0; i<jObject.length; i++) {
            v.addElement(jObject[i].toString());
        }
        //emulador NetBeans file:///root1/
        //writeFile = "file:///C:/"+fichero;
        writeFile = System.getProperty("fileconn.dir.memorycard")+fichero;
        //writeFile = "file:///root1/"+fichero;
        try {
            OutputStream os;
            //FileConnection fc = (FileConnection) Connector.open(writeFile, Connector.READ_WRITE);
            FileConnection fc = (FileConnection) Connector.open(writeFile, Connector.READ_WRITE);
            if (!fc.exists())
                fc.create();
            if (sobreescribir)
                os = fc.openOutputStream();
            else
                os = fc.openOutputStream(fc.fileSize());//empieza a escribir al final del fichero
            PrintStream ps = new PrintStream(os);
            for (int i=0; i<v.size(); i++)
                ps.println(v.elementAt(i).toString());
            fc.close();
            exito = 1;
        }
        catch (IOException e){
            e.printStackTrace();
            exito = -1;
        }
        return exito;
    }

    /**
     * Constructor de la clase saveData. Guarda un json en forma de String en un fichero de texto.
     * Tiene la opción de sobrescribir el fichero o escribir al final de éste.
     * @param fichero nombre del fichero donde se va a guardar la información.
     * @param json objeto que se va a guardar en el fichero.
     * @param ruta ruta del directorio del fichero que se va a leer.
     * @param sobreescribir si es true se sobreescribirá el contenido fichero. Si es false se respetará el contenido
     * y la información se añadirá al final del fichero.
     * @return código de retorno del constructor. Si finaliza con éxito devuelve 1.
     */
    public int saveData(String fichero, String json, final boolean sobreescribir, String ruta) {
        final String writeFile, writeContents;
        try {
            writeFile = "file://"+ruta+fichero;
            writeContents = json;
            Thread t = new Thread(){
                public void run(){
                    try {
                        OutputStream os;
                        FileConnection fc = (FileConnection) Connector.open(writeFile, Connector.WRITE);
                        if (!fc.exists())
                            fc.create();
                        if (sobreescribir) {
                            os = fc.openOutputStream();//empieza a escribir al principio del fichero
                        }
                        else
                            os = fc.openOutputStream(fc.fileSize());//le decimos que empiece al final
                        PrintStream ps = new PrintStream(os);
                        ps.println(writeContents);
                        fc.close();
                        exito = 1;
                    } catch (IOException e){
                        e.printStackTrace();
                        exito = -1;
                    } catch (SecurityException e){
                        e.printStackTrace();
                        exito = -2;
                    }
                }
            };
            t.start();
        } catch (Exception e) {
            e.printStackTrace();
            exito = -1;
        }
        return exito;
    }

    /**
     * Constructor de la clase saveData. Guarda un vector de json en forma de String en un fichero de texto.
     * Tiene la opción de sobrescribir el fichero o escribir al final de éste.
     * @param fichero nombre del fichero donde se va a guardar la información.
     * @param ruta ruta del directorio del fichero que se va a leer.
     * @param sobreescribir si es true se sobreescribirá el contenido fichero. Si es false se respetará el contenido
     * y la información se añadirá al final del fichero.
     * @param vjson vector cuyos elementos serán guardados en el fichero separados por saltos de línea.
     * @return código de retorno del constructor. Si finaliza con éxito devuelve 1.
     */
    public int saveData(String fichero, Vector vjson, final boolean sobreescribir, String ruta) {
        final String writeFile;
        writeFile = "file://"+ruta+fichero;
        try {
            OutputStream os;
            FileConnection fc = (FileConnection) Connector.open(writeFile, Connector.WRITE);
            if (!fc.exists())
                fc.create();
            if (sobreescribir) {
                fc.truncate(0);//elimina el contenido del fichero registros
                os = fc.openOutputStream();
                vjson.trimToSize();
                fc.fileSize();
            }
            else
                os = fc.openOutputStream(fc.fileSize());//para que empiece a escribir al final del fichero
            PrintStream ps = new PrintStream(os);
            for (int i=0; i<vjson.size(); i++)
                ps.println(vjson.elementAt(i).toString());
            fc.close();
            exito = 1;
        }
        catch (IOException e) {
            e.printStackTrace();
            exito = -1;
        }
        catch (SecurityException se) {
            se.printStackTrace();
            exito = -2;
        }
        return exito;
    }

    /**
     * Constructor de la clase saveData. Guarda un array de objetos JSON en un fichero de texto.
     * Tiene la opción de sobrescribir el fichero o escribir al final de éste.
     * @param fichero nombre del fichero donde se va a guardar la información.
     * @param jObject array de objetos JSON cuyos elementos serán guardados en el fichero separados por saltos de línea.
     * @param ruta ruta del directorio del fichero que se va a leer.
     * @param sobreescribir si es true se sobreescribirá el contenido fichero. Si es false se respetará el contenido
     * y la información se añadirá al final del fichero.
     * @return código de retorno del constructor. Si finaliza con éxito devuelve 1.
     */
    public int saveData(String fichero, JSONObject[] jObject, final boolean sobreescribir, String ruta) {
        final String writeFile;
        Vector v = new Vector();
        for (int i=0; i<jObject.length; i++) {
            v.addElement(jObject[i].toString());
        }
        writeFile = "file://"+ruta+fichero;
        try {
            OutputStream os;
            FileConnection fc = (FileConnection) Connector.open(writeFile, Connector.WRITE);
            if (!fc.exists())
                fc.create();
            if (sobreescribir)
                os = fc.openOutputStream();
            else
                os = fc.openOutputStream(fc.fileSize());//empieza a escribir al final del fichero
            PrintStream ps = new PrintStream(os);
            for (int i=0; i<v.size(); i++)
                ps.println(v.elementAt(i).toString());
            fc.close();
            exito = 1;
        }
        catch (IOException e){
            e.printStackTrace();
            exito = -1;
        }
        return exito;
    }

    /**
     * Elimina el fichero
     * @param fichero nombre del fichero que se desea elminar
     */
    public void delete(String fichero) {
        try {
            String delFile = System.getProperty("fileconn.dir.memorycard")+fichero;
            FileConnection fc = (FileConnection) Connector.open(delFile, Connector.READ_WRITE);
            fc.delete();
            fc.close();
        }
        catch (IOException e) {
            e.printStackTrace();
        }
    }
}